@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c,v 1.65 2006/01/05 20:06:23 paolo Exp $ 
+ * $Id: gedit-view.c,v 1.68 2006/02/12 17:30:50 pborelli Exp $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -64,6 +64,9 @@ typedef enum
 struct _GeditViewPrivate
 {
 	GtkTooltips *tooltips;
+
+	/* idle hack to make open-at-line work */
+	guint        scroll_idle;
 	
 	SearchMode   search_mode;
 	
@@ -316,6 +319,7 @@ scroll_to_cursor_on_init (GeditView *view)
 {
 	gedit_view_scroll_to_cursor (view);
 
+	view->priv->scroll_idle = 0;
 	return FALSE;
 }
 
@@ -380,7 +384,7 @@ gedit_view_init (GeditView *view)
 	 * possible: see bug #172277 and bug #311728.
 	 * So we need to do this in an idle handler.
 	 */
-	g_idle_add ((GSourceFunc) scroll_to_cursor_on_init, view);
+	view->priv->scroll_idle = g_idle_add ((GSourceFunc) scroll_to_cursor_on_init, view);
 	
 	view->priv->typeselect_flush_timeout = 0;	
 	view->priv->wrap_around = TRUE;		   
@@ -420,6 +424,9 @@ gedit_view_finalize (GObject *object)
 	
 	if (view->priv->tooltips != NULL)
 		g_object_unref (view->priv->tooltips);
+
+	if (view->priv->scroll_idle > 0)
+		g_source_remove (view->priv->scroll_idle);
 		
 	(* G_OBJECT_CLASS (gedit_view_parent_class)->finalize) (object);
 }
@@ -487,7 +494,8 @@ gedit_view_new (GeditDocument *doc)
 void
 gedit_view_cut_clipboard (GeditView *view)
 {
-	GtkTextBuffer *buffer = NULL;
+	GtkTextBuffer *buffer;
+	GtkClipboard *clipboard;
 
 	gedit_debug (DEBUG_VIEW);
 
@@ -496,9 +504,12 @@ gedit_view_cut_clipboard (GeditView *view)
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 	g_return_if_fail (buffer != NULL);
 
+	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view),
+					      GDK_SELECTION_CLIPBOARD);
+
 	/* FIXME: what is default editability of a buffer? */
   	gtk_text_buffer_cut_clipboard (buffer,
-  				       gtk_clipboard_get (GDK_NONE),
+  				       clipboard,
 				       !gedit_document_get_readonly (
 				       		GEDIT_DOCUMENT (buffer)));
   	
@@ -513,7 +524,8 @@ gedit_view_cut_clipboard (GeditView *view)
 void
 gedit_view_copy_clipboard (GeditView *view)
 {
-	GtkTextBuffer *buffer = NULL;
+	GtkTextBuffer *buffer;
+	GtkClipboard *clipboard;
 
 	gedit_debug (DEBUG_VIEW);
 
@@ -522,8 +534,10 @@ gedit_view_copy_clipboard (GeditView *view)
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 	g_return_if_fail (buffer != NULL);
 
-  	gtk_text_buffer_copy_clipboard (buffer,
-  					gtk_clipboard_get (GDK_NONE));
+	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view),
+					      GDK_SELECTION_CLIPBOARD);
+
+  	gtk_text_buffer_copy_clipboard (buffer, clipboard);
 
 	/* on copy do not scroll, we are already on screen */
 }
@@ -531,7 +545,8 @@ gedit_view_copy_clipboard (GeditView *view)
 void
 gedit_view_paste_clipboard (GeditView *view)
 {
-  	GtkTextBuffer *buffer = NULL;
+  	GtkTextBuffer *buffer;
+	GtkClipboard *clipboard;
 
 	gedit_debug (DEBUG_VIEW);
 
@@ -540,9 +555,12 @@ gedit_view_paste_clipboard (GeditView *view)
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 	g_return_if_fail (buffer != NULL);
 
+	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view),
+					      GDK_SELECTION_CLIPBOARD);
+
 	/* FIXME: what is default editability of a buffer? */
   	gtk_text_buffer_paste_clipboard (buffer,
-					 gtk_clipboard_get (GDK_NONE),
+					 clipboard,
 					 NULL,
 					 !gedit_document_get_readonly (
 						GEDIT_DOCUMENT (buffer)));
@@ -1042,6 +1060,7 @@ hide_search_window (GeditView *view, gboolean cancel)
 
 	/* send focus-in event */
 	send_focus_change (GTK_WIDGET (view->priv->search_entry), FALSE);
+	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view), TRUE);
 	gtk_widget_hide (view->priv->search_window);
 	
 	if (cancel)
@@ -1717,16 +1736,16 @@ start_interactive_search_real (GeditView *view)
 		return FALSE;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	
+
 	if (view->priv->search_mode == SEARCH)
 		gtk_text_buffer_get_selection_bounds (buffer, &view->priv->start_search_iter, NULL);
 	else
 		gtk_text_buffer_get_iter_at_mark (buffer,
 						  &view->priv->start_search_iter,
 						  gtk_text_buffer_get_insert (buffer));
-					  		
+
 	ensure_search_window (view);
-		
+
 	/* done, show it */
 	update_search_window_position (view);
 	gtk_widget_show (view->priv->search_window);
@@ -1734,18 +1753,20 @@ start_interactive_search_real (GeditView *view)
 	if (view->priv->search_entry_changed_id == 0)
 	{
       		view->priv->search_entry_changed_id =
-			g_signal_connect (view->priv->search_entry, "changed",
+			g_signal_connect (view->priv->search_entry,
+					  "changed",
 					  G_CALLBACK (search_init),
 					  view);
 	}
-	
+
 	init_search_entry (view);
 
 	view->priv->typeselect_flush_timeout =  
 		g_timeout_add (GEDIT_VIEW_SEARCH_DIALOG_TIMEOUT,
 		   	       (GSourceFunc) search_entry_flush_timeout,
 		   	       view);
-		   
+
+	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view), FALSE);
 	gtk_widget_grab_focus (view->priv->search_entry);
 	
 	send_focus_change (view->priv->search_entry, TRUE);
