@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gedit-spell-plugin.c,v 1.2 2005/12/12 17:28:02 pborelli Exp $
+ * $Id: gedit-spell-plugin.c,v 1.4 2006/01/02 18:40:05 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -75,7 +75,7 @@ static const GtkActionEntry action_entries[] =
 
 	{ "ConfigSpell",
 	  GNOME_STOCK_BOOK_BLUE,
-	  N_("Set _Language"),
+	  N_("Set _Language..."),
 	  NULL,
 	  N_("Set the language of the current document"),
 	  G_CALLBACK (set_language_cb)
@@ -476,8 +476,9 @@ get_next_misspelled_word (GeditView *view)
 		if (!goto_next_word (doc))
 			return NULL;
 
+		/* may return null if we reached the end of the selection */
 		word = get_current_word (doc, &start, &end);
-		g_return_val_if_fail (word != NULL, NULL);
+			return NULL;
 
 		gedit_debug_message (DEBUG_PLUGINS, "Word to check: %s", word);
 	}
