@@ -1,9 +1,8 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gedit-taglist-plugin-parser.c
- * This file is part of the gedit taglist plugin
+ * This file is part of gedit
  *
- * Copyright (C) 2002 Paolo Maggi 
+ * Copyright (C) 2002-2005 - Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,31 +21,44 @@
  */
  
 /*
- * Modified by the gedit Team, 2002. See the AUTHORS file for a 
+ * Modified by the gedit Team, 2002-2005. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
+ *
+ * $Id: gedit-taglist-plugin-parser.c,v 1.11 2005/12/12 17:28:02 pborelli Exp $
  */
 
-#include <libxml/parser.h>
-#include <glib.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <dirent.h> 
 #include <string.h>
 #include <errno.h>
 
+#include <libxml/parser.h>
+#include <glib.h>
+#include <glib/gi18n.h>
+
 #include <libgnome/gnome-util.h>
 
-#include <gedit-debug.h>
+#include <gedit/gedit-debug.h>
 
 #include "gedit-taglist-plugin-parser.h"
 
-#define USER_GEDIT_TAGLIST_PLUGIN_LOCATION ".gedit2/plugins/taglist/"
+// FIXME
+#define USER_GEDIT_TAGLIST_PLUGIN_LOCATION ".gedit-2/plugins/taglist/"
 
 TagList *taglist = NULL;
+static gint taglist_ref_count = 0;
 
 static gboolean	 parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
 static gboolean	 parse_tag_group (TagGroup *tg, const gchar *fn, 
 				  xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
+static TagGroup* get_tag_group (const gchar* filename, xmlDocPtr doc, 
+				xmlNsPtr ns, xmlNodePtr cur);
+static TagList* lookup_best_lang (TagList *taglist, const gchar *filename, 
+				xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
 static TagList 	*parse_taglist_file (const gchar* filename);
 static TagList  *parse_taglist_dir (const gchar *dir);
 
@@ -57,7 +69,7 @@ static gboolean
 parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) 
 {
 	/*
-	gedit_debug (DEBUG_PLUGINS, "  Tag name: %s", tag->name);
+	gedit_debug_message (DEBUG_PLUGINS, "  Tag name: %s", tag->name);
 	*/
 	/* We don't care what the top level element name is */
 	cur = cur->xmlChildrenNode;
@@ -69,7 +81,7 @@ parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 		{			
 			tag->begin = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
 			/*
-			gedit_debug (DEBUG_PLUGINS, "    - Begin: %s", tag->begin);
+			gedit_debug_message (DEBUG_PLUGINS, "    - Begin: %s", tag->begin);
 			*/
 		}
 
@@ -78,7 +90,7 @@ parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 		{
 			tag->end = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
 			/*
-			gedit_debug (DEBUG_PLUGINS, "    - End: %s", tag->end);
+			gedit_debug_message (DEBUG_PLUGINS, "    - End: %s", tag->end);
 			*/
 		}
 
@@ -95,7 +107,7 @@ static gboolean
 parse_tag_group (TagGroup *tg, const gchar* fn, xmlDocPtr doc, 
 		 xmlNsPtr ns, xmlNodePtr cur) 
 {
-	gedit_debug (DEBUG_PLUGINS, "Parse TagGroup: %s", tg->name);
+	gedit_debug_message (DEBUG_PLUGINS, "Parse TagGroup: %s", tg->name);
 
 	/* We don't care what the top level element name is */
     	cur = cur->xmlChildrenNode;
@@ -157,6 +169,214 @@ parse_tag_group (TagGroup *tg, const gchar* fn, xmlDocPtr doc,
 }
 
 
+static TagGroup*
+get_tag_group (const gchar* filename, xmlDocPtr doc, 
+		 xmlNsPtr ns, xmlNodePtr cur) 
+{
+			TagGroup *tag_group;
+
+
+			tag_group = g_new0 (TagGroup, 1);
+
+			/* Get TagGroup name */
+			tag_group->name = xmlGetProp (cur, (const xmlChar *) "name");
+
+			if (tag_group->name == NULL)
+			{
+				/* Error: No name */
+				g_warning ("The tag list file '%s' is of the wrong type, "
+				   "TagGroup without name.", filename);
+
+				g_free (tag_group);
+			}
+			else
+			{
+				/* Name found */
+				gboolean exists = FALSE;
+				GList *t = taglist->tag_groups;
+				
+				/* Check if the tag group already exists */
+				while (t && !exists)
+				{
+					gchar *tgn = (gchar*)((TagGroup*)(t->data))->name;
+					
+					if (strcmp (tgn, (gchar*)tag_group->name) == 0)
+					{
+						gedit_debug_message (DEBUG_PLUGINS, 
+							     "Tag group '%s' already exists.", tgn);
+						
+						exists = TRUE;
+
+						free_tag_group (tag_group);
+					}
+					
+					t = g_list_next (t);		
+				}
+
+				if (!exists)
+				{				
+					/* Parse tag group */
+					if (parse_tag_group (tag_group, filename, doc, ns, cur))
+					{
+						return tag_group;
+					}
+					else
+					{
+						/* Error parsing TagGroup */
+						g_warning ("The tag list file '%s' is of the wrong type, "
+				   			   "error parsing TagGroup '%s'.", 
+							   filename, tag_group->name);
+
+						free_tag_group (tag_group);
+					}
+				}
+			}
+	return NULL;
+}
+
+
+/* 
+ *  tags file is localized by intltool-merge below.
+ *
+ *      <gedit:TagGroup name="XSLT - Elements">
+ *      </gedit:TagGroup>
+ *      <gedit:TagGroup xml:lang="am" name="LOCALIZED TEXT">
+ *      </gedit:TagGroup>
+ *      <gedit:TagGroup xml:lang="ar" name="LOCALIZED TEXT">
+ *      </gedit:TagGroup>
+ *      .....
+ *      <gedit:TagGroup name="XSLT - Functions">
+ *      </gedit:TagGroup>
+ *      .....
+ *  Therefore need to pick up the best lang on the current locale.
+ */
+static TagList*
+lookup_best_lang (TagList *taglist, const gchar *filename, 
+		xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) 
+{
+
+	TagGroup *best_tag_group = NULL;
+	TagGroup *tag_group;
+	gint best_lanking = -1;
+
+	/*
+	 * Walk the tree.
+	 *
+	 * First level we expect a list TagGroup 
+	 */
+	cur = cur->xmlChildrenNode;
+	
+	while (cur != NULL)
+     	{ 	
+		if ((xmlStrcmp (cur->name, (const xmlChar *) "TagGroup")) || (cur->ns != ns)) 
+		{
+			g_warning ("The tag list file '%s' is of the wrong type, "
+				   "was '%s', 'TagGroup' expected.", filename, cur->name);
+			xmlFreeDoc (doc);
+
+			return taglist;
+		}
+		else
+		{
+			const char * const *langs_pointer;
+			gchar *lang;
+			gint cur_lanking;
+			gint i;
+
+			langs_pointer = g_get_language_names ();
+
+			lang = (gchar*) xmlGetProp (cur, (const xmlChar*) "lang");
+			cur_lanking = 1;
+
+			/* 
+			 * When found a new TagGroup, append the best 
+			 * tag_group to taglist. In the current intltool-merge, 
+			 * the first section is the default lang NULL.
+			 */
+			if (lang == NULL) {
+				if (best_tag_group != NULL) {
+					taglist->tag_groups = 
+					g_list_append (taglist->tag_groups, best_tag_group);
+				}
+
+				best_tag_group = NULL;
+				best_lanking = -1;
+			}
+
+			/* 
+			 * If already find the best TagGroup on the current 
+			 * locale, ignore the logic.
+			 */
+			if (best_lanking != -1 && best_lanking <= cur_lanking) {
+				cur = cur->next;
+				continue;
+			}
+
+			/* try to find the best lang */
+			for (i = 0; langs_pointer[i] != NULL; i++)
+			{
+				const gchar *best_lang = langs_pointer[i];
+
+				/*
+				 * if launch on C, POSIX locale or does 
+				 * not find the best lang on the current locale,
+				 * this is called.
+				 * g_get_language_names returns lang 
+				 * lists with C locale.
+				 */
+				if (lang == NULL &&
+				    (!g_ascii_strcasecmp (best_lang, "C") || 
+				     !g_ascii_strcasecmp (best_lang, "POSIX")))
+				{
+					tag_group = get_tag_group (filename, doc, ns, cur);
+					if (tag_group != NULL)
+					{
+						if (best_tag_group !=NULL) 
+							free_tag_group (best_tag_group);
+						best_lanking = cur_lanking;
+						best_tag_group = tag_group;
+					}
+				}
+
+				/* if it is possible the best lang is not C */
+				else if (lang == NULL)
+				{
+					cur_lanking++;
+					continue;
+				}
+
+				/* if the best lang is found */
+				else if (!g_ascii_strcasecmp (best_lang, lang))
+				{
+					tag_group = get_tag_group (filename, doc, ns, cur);
+					if (tag_group != NULL)
+					{
+						if (best_tag_group !=NULL) 
+							free_tag_group (best_tag_group);
+						best_lanking = cur_lanking;
+						best_tag_group = tag_group;
+					}
+				}
+
+				cur_lanking++;
+			}
+
+			if (lang) g_free (lang);
+		} /* End of else */
+	
+		cur = cur->next;
+    	} /* End of while (cur != NULL) */
+
+	/* Append TagGroup to TagList */
+	if (best_tag_group != NULL) {
+		taglist->tag_groups = 
+			g_list_append (taglist->tag_groups, best_tag_group);
+	}
+
+	return taglist;
+}
+
+
 static TagList *
 parse_taglist_file (const gchar* filename)
 {
@@ -165,7 +385,7 @@ parse_taglist_file (const gchar* filename)
 	xmlNsPtr ns;
 	xmlNodePtr cur;
 
-	gedit_debug (DEBUG_PLUGINS, "Parse file: %s", filename);
+	gedit_debug_message (DEBUG_PLUGINS, "Parse file: %s", filename);
 
 	xmlKeepBlanksDefault (0);
 
@@ -221,92 +441,11 @@ parse_taglist_file (const gchar* filename)
 	if (taglist == NULL)
 		taglist = g_new0 (TagList, 1);
 	
-	/*
-	 * Walk the tree.
-	 *
-	 * First level we expect a list TagGroup 
-	 */
-	cur = cur->xmlChildrenNode;
-	
-	while (cur != NULL)
-     	{ 	
-		if ((xmlStrcmp (cur->name, (const xmlChar *) "TagGroup")) || (cur->ns != ns)) 
-		{
-			g_warning ("The tag list file '%s' is of the wrong type, "
-				   "was '%s', 'TagGroup' expected.", filename, cur->name);
-			xmlFreeDoc (doc);
-		
-			return taglist;
-		}
-		else
-		{
-			TagGroup *tag_group;
-
-			tag_group = g_new0 (TagGroup, 1);
-
-			/* Get TagGroup name */
-			tag_group->name = xmlGetProp (cur, (const xmlChar *) "name");
-
-			if (tag_group->name == NULL)
-			{
-				/* Error: No name */
-				g_warning ("The tag list file '%s' is of the wrong type, "
-				   "TagGroup without name.", filename);
-
-				g_free (tag_group);
-			}
-			else
-			{
-				/* Name found */
-				gboolean exists = FALSE;
-				GList *t = taglist->tag_groups;
-				
-				/* Check if the tag group already exists */
-				while (t && !exists)
-				{
-					gchar *tgn = ((TagGroup*)(t->data))->name;
-					
-					if (strcmp (tgn, tag_group->name) == 0)
-					{
-						gedit_debug (DEBUG_PLUGINS, 
-							     "Tag group '%s' already exists.", tgn);
-						
-						exists = TRUE;
-
-						free_tag_group (tag_group);
-					}
-					
-					t = g_list_next (t);		
-				}
-
-				if (!exists)
-				{				
-					/* Parse tag group */
-					if (parse_tag_group (tag_group, filename, doc, ns, cur))
-					{
-						/* Append TagGroup to TagList */
-						taglist->tag_groups = 
-							g_list_append (taglist->tag_groups, tag_group);
-					}
-					else
-					{
-						/* Error parsing TagGroup */
-						g_warning ("The tag list file '%s' is of the wrong type, "
-				   			   "error parsing TagGroup '%s'.", 
-							   filename, tag_group->name);
-
-						free_tag_group (tag_group);
-					}
-				}
-			}
-		}
-	
-		cur = cur->next;
-    	}
+	taglist = lookup_best_lang (taglist, filename, doc, ns, cur);
 
 	xmlFreeDoc (doc);
 
-	gedit_debug (DEBUG_PLUGINS, "END");
+	gedit_debug_message (DEBUG_PLUGINS, "END");
 
 	return taglist;
 }
@@ -316,7 +455,7 @@ static void
 free_tag (Tag *tag)
 {
 	/*
-	gedit_debug (DEBUG_PLUGINS, "Tag: %s", tag->name);
+	gedit_debug_message (DEBUG_PLUGINS, "Tag: %s", tag->name);
 	*/
 	g_return_if_fail (tag != NULL);
 
@@ -334,7 +473,7 @@ free_tag (Tag *tag)
 static void
 free_tag_group (TagGroup *tag_group)
 {
-	gedit_debug (DEBUG_PLUGINS, "Tag group: %s", tag_group->name);
+	gedit_debug_message (DEBUG_PLUGINS, "Tag group: %s", tag_group->name);
 
 	g_return_if_fail (tag_group != NULL);
 
@@ -351,17 +490,23 @@ free_tag_group (TagGroup *tag_group)
 	
 	g_free (tag_group);
 
-	gedit_debug (DEBUG_PLUGINS, "END");
+	gedit_debug_message (DEBUG_PLUGINS, "END");
 }
 
 void
 free_taglist (void)
 {
-	gedit_debug (DEBUG_PLUGINS, "");
+	gedit_debug_message (DEBUG_PLUGINS, "ref_count: %d", taglist_ref_count);
+	
+	g_return_if_fail (taglist_ref_count > 0);
 
+	--taglist_ref_count;
+	if (taglist_ref_count > 0)
+		return;
+		
 	if (taglist == NULL)
 		return;
-	
+		
 	while (taglist->tag_groups)
 	{
 		free_tag_group ((TagGroup *)taglist->tag_groups->data);
@@ -375,7 +520,7 @@ free_taglist (void)
 
 	taglist = NULL;
 
-	gedit_debug (DEBUG_PLUGINS, "END");
+	gedit_debug_message (DEBUG_PLUGINS, "Really freed");
 }
 
 static TagList * 
@@ -384,13 +529,13 @@ parse_taglist_dir (const gchar *dir)
 	DIR *d;
 	struct dirent *e;
 
-	gedit_debug (DEBUG_PLUGINS, "DIR: %s", dir);
+	gedit_debug_message (DEBUG_PLUGINS, "DIR: %s", dir);
 
 	d = opendir (dir);
 	
 	if (d == NULL)
 	{		
-		gedit_debug (DEBUG_PLUGINS, "%s", strerror (errno));
+		gedit_debug_message (DEBUG_PLUGINS, "%s", strerror (errno));
 		return taglist;
 	}
 	
@@ -412,10 +557,15 @@ TagList* create_taglist (void)
 {
 	gchar const * const home = g_get_home_dir ();
 
-	gedit_debug (DEBUG_PLUGINS, "");
-
-	g_return_val_if_fail (taglist == NULL, taglist);
+	gedit_debug_message (DEBUG_PLUGINS, "ref_count: %d", taglist_ref_count);
 
+	if (taglist_ref_count > 0)
+	{		
+		++taglist_ref_count;
+		
+		return taglist;
+	}
+		
 	/* load user's taglists */
 	if (home != NULL)
 	{
@@ -430,5 +580,8 @@ TagList* create_taglist (void)
 	/* load system's taglists */	
 	parse_taglist_dir (GEDIT_TAGLIST_DIR);
 
+	++taglist_ref_count;
+	g_return_val_if_fail (taglist_ref_count == 1, taglist);
+	
 	return taglist;
 }
